<?php

namespace Drupal\wse_config\Cache;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\ChainedFastBackend;
use Drupal\wse_config\WorkspaceIdDetector;

/**
 * Defines a chained fast backend that omits fast caching inside workspaces.
 */
class WseChainedFastBackend extends ChainedFastBackend {

  /**
   * The workspace ID detector.
   *
   * @var \Drupal\wse_config\WorkspaceIdDetector
   */
  protected $workspaceIdDetector;

  /**
   * Constructs a ChainedFastBackend object.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $consistent_backend
   *   The consistent cache backend.
   * @param \Drupal\Core\Cache\CacheBackendInterface $fast_backend
   *   The fast cache backend.
   * @param string $bin
   *   The cache bin for which the object is created.
   */
  public function __construct(CacheBackendInterface $consistent_backend, CacheBackendInterface $fast_backend, $bin, WorkspaceIdDetector $workspace_id_detector) {
    parent::__construct($consistent_backend, $fast_backend, $bin);
    $this->workspaceIdDetector = $workspace_id_detector;
  }

  /**
   * Gets the last write timestamp.
   */
  protected function getLastWriteTimestamp() {
    $workspace_id = $this->workspaceIdDetector->getActiveWorkspaceId();
    if ($workspace_id) {
      return 0;
    }

    if ($this->lastWriteTimestamp === NULL) {
      $cache = $this->consistentBackend->get(ChainedFastBackend::LAST_WRITE_TIMESTAMP_PREFIX . $this->bin);
      $this->lastWriteTimestamp = $cache ? $cache->data : 0;
    }
    return $this->lastWriteTimestamp;
  }

}
