<?php

namespace Drupal\wse_lb;

use Drupal\layout_builder\Plugin\Block\InlineBlock;

/**
 * Provides an override for InlineBlocks that makes them workspaces trackable.
 *
 * This is done by systematically removing all references to block revision ID
 * and instead depending on workspaces to get the appropriate revision for us.
 */
class InlineBlockOverride extends InlineBlock {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'block_id' => NULL,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function saveBlockContent($new_revision = FALSE, $duplicate_block = FALSE) {
    /** @var \Drupal\block_content\BlockContentInterface $block */
    $block = NULL;
    if (!empty($this->configuration['block_serialized'])) {
      $block = unserialize($this->configuration['block_serialized']);
    }
    if ($duplicate_block) {
      if (empty($block) && !empty($this->configuration['block_revision_id'])) {
        $block = $this->entityTypeManager->getStorage('block_content')->loadRevision($this->configuration['block_revision_id']);
      }
      if ($block) {
        $block = $block->createDuplicate();
      }
    }

    if ($block) {
      // Since the content block is only set if it was unserialized, the flag
      // will only effect blocks which were modified or serialized originally.
      if ($new_revision) {
        $block->setNewRevision();
      }
      $block->save();

      // Reload the block to ensure that we store a possible workspace-specific
      // revision ID.
      $block = $this->entityTypeManager->getStorage('block_content')->load($block->id());

      $this->configuration['block_id'] = $block->id();
      $this->configuration['block_revision_id'] = $block->getRevisionId();
      $this->configuration['block_serialized'] = NULL;
    }
  }

}
