<?php

namespace Drupal\wse_menu\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionLogEntityTrait;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\wse_menu\WseMenuTreeInterface;

/**
 * Defines the wse_menu_tree entity type.
 *
 * @ContentEntityType(
 *   id = "wse_menu_tree",
 *   label = @Translation("Workspace menu tree"),
 *   label_collection = @Translation("Workspace menu trees"),
 *   base_table = "wse_menu_tree",
 *   revision_table = "wse_menu_tree_revision",
 *   admin_permission = "administer menu",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "langcode" = "langcode",
 *     "uuid" = "uuid",
 *     "published" = "status",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log"
 *   },
 *   links = {
 *     "canonical" = "/admin/config/workflow/workspaces/wse_menu/{wse_menu_tree}",
 *   },
 * )
 */
class WseMenuTree extends EditorialContentEntityBase implements WseMenuTreeInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;
  use RevisionLogEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function getDefinitions(): array {
    $data = $this->get('definitions')->value;
    return (array) json_decode($data ?? '', TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function setDefinitions(array $data): static {
    $this->set('definitions', json_encode($data));
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return t('Menu tree');
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    /** @var \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager */
    $workspace_manager = \Drupal::service('workspaces.manager');
    if (!$workspace_manager->hasActiveWorkspace()) {
      // Ensure that the default revision of this entity doesn't store any
      // definition.
      $this->set('definitions', NULL);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the workspace menu tree was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the workspace menu tree was last edited.'));

    $fields['definitions'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('JSON encoded data of all menu link plugin definitions'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    return $fields;
  }

}
