<?php

namespace Drupal\wse_menu;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;

/**
 * Provides an interface defining a workspace menu tree entity type.
 */
interface WseMenuTreeInterface extends ContentEntityInterface, EntityChangedInterface {

  /**
   * Gets the workspace-specific menu link definitions.
   *
   * @return array
   */
  public function getDefinitions(): array;

  /**
   * Sets the workspace-specific menu link definitions.
   *
   * @param array $data
   *   An array of menu link definitions, as returned by
   *   \Drupal\wse_menu\WseMenuTreeStorage::loadMultiple().
   *
   * @return $this
   */
  public function setDefinitions(array $data): static;

  /**
   * Gets the workspace menu tree creation timestamp.
   *
   * @return int
   *   Creation timestamp of the workspace menu tree.
   */
  public function getCreatedTime();

  /**
   * Sets the workspace menu tree creation timestamp.
   *
   * @param int $timestamp
   *   The workspace menu tree creation timestamp.
   *
   * @return \Drupal\wse_menu\WseMenuTreeInterface
   *   The called workspace menu tree entity.
   */
  public function setCreatedTime($timestamp);

}
