<?php

namespace Drupal\wse\Controller;

use Drupal\Core\Entity\Controller\VersionHistoryController;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Entity\RevisionableStorageInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\workspaces\WorkspaceManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Overrides core's VersionHistoryController.
 */
class WseVersionHistoryController extends VersionHistoryController {

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected WorkspaceManagerInterface $workspaceManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->workspaceManager = $container->get('workspaces.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function loadRevisions(RevisionableInterface $entity) {
    $entityType = $entity->getEntityType();
    $translatable = $entityType->isTranslatable();
    $entityStorage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    assert($entityStorage instanceof RevisionableStorageInterface);

    $query = $entityStorage->getQuery()
      ->accessCheck(FALSE)
      ->allRevisions()
      ->condition($entityType->getKey('id'), $entity->id())
      ->sort($entityType->getKey('revision'), 'DESC')
      ->pager(self::REVISIONS_PER_PAGE);

    // When a workspace is active, filter the revisions to those created in that
    // workspace.
    if ($this->workspaceManager->hasActiveWorkspace()) {
      $active_workspace_id = $this->workspaceManager->getActiveWorkspace()->id();
      $query->condition($entity->getEntityType()->getRevisionMetadataKey('workspace'), $active_workspace_id);
    }
    else {
      // Otherwise show only revisions that were not created in a workspace.
      $query->notExists($entity->getEntityType()->getRevisionMetadataKey('workspace'));
    }

    $result = $query->execute();

    $currentLangcode = $this->languageManager
      ->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)
      ->getId();
    foreach ($entityStorage->loadMultipleRevisions(array_keys($result)) as $revision) {
      // Only show revisions that are affected by the language that is being
      // displayed.
      if (!$translatable || ($revision->hasTranslation($currentLangcode) && $revision->getTranslation($currentLangcode)->isRevisionTranslationAffected())) {
        yield ($translatable ? $revision->getTranslation($currentLangcode) : $revision);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function revisionOverview(RevisionableInterface $entity): array {
    $build = parent::revisionOverview($entity);

    if ($this->workspaceManager->hasActiveWorkspace()) {
      $build['entity_revisions_table']['#empty'] = $this->t('There are no revisions yet in this workspace.');
      $build['#cache']['contexts'][] = 'workspace';
    }

      return $build;
  }

}
