<?php

namespace Drupal\wse\Core\EventSubscriber;

use Drupal\workspaces\Event\WorkspaceSafeFormEvent;
use Drupal\workspaces\Form\WorkspaceFormInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Provides an event subscriber for setting default workspace-safe forms.
 */
class DefaultWorkspaceSafeForms implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[WorkspaceSafeFormEvent::class][] = 'onWorkspaceSafeForm';
    return $events;
  }

  /**
   * Marks workspaces internal forms as workspace-safe.
   *
   * @param \Drupal\workspaces\Event\WorkspaceSafeFormEvent $event
   *   The workspace-safe form event.
   */
  public function onWorkspaceSafeForm(WorkspaceSafeFormEvent $event): void {
    // Allow a few forms that we know are safe to submit.
    if ($event->getFormState()->getFormObject() instanceof WorkspaceFormInterface) {
      $event->setSafe(TRUE);
      $event->stopPropagation();
    }
  }

}
