<?php

namespace Drupal\wse\EventSubscriber;

use Drupal\workspaces\Event\WorkspaceSafeFormEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Provides an event subscriber for setting workspace-safe forms.
 */
class WseWorkspaceSafeForms implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[WorkspaceSafeFormEvent::class][] = 'onWorkspaceSafeForm';
    return $events;
  }

  /**
   * Marks various forms as workspace-safe.
   *
   * @param \Drupal\workspaces\Event\WorkspaceSafeFormEvent $event
   *   The workspace-safe form event.
   */
  public function onWorkspaceSafeForm(WorkspaceSafeFormEvent $event): void {
    $safe_form_ids = [
      'node_revision_revert_confirm',
    ];
    // Allow a few forms that we know are safe to submit.
    if (in_array($event->getFormId(), $safe_form_ids, TRUE)) {
      $event->setSafe(TRUE);
      $event->stopPropagation();
    }
  }

}
