<?php

namespace Drupal\wse;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceProviderBase;
use Drupal\wse\Controller\WseHtmlEntityFormController;
use Drupal\wse\Diff\WseDiffEntityParser;
use Drupal\wse\Negotiator\WseQueryParameterWorkspaceNegotiator;
use Symfony\Component\DependencyInjection\Reference;

/**
 * Defines a service provider for the Workspaces module.
 */
class WseServiceProvider extends ServiceProviderBase {

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container) {
    // We cannot use the module handler as the container is not yet compiled.
    // @see \Drupal\Core\DrupalKernel::compileContainer()
    $modules = $container->getParameter('container.modules');

    if (isset($modules['diff'])) {
      $container->register('diff.entity_parser.wse', WseDiffEntityParser::class)
        ->setDecoratedService('diff.entity_parser')
        ->setPublic(FALSE)
        ->addArgument(new Reference('plugin.manager.diff.builder'))
        ->addArgument(new Reference('config.factory'));
    }


    $container->register('Drupal\workspaces\EntityAccess', \Drupal\wse\Core\EntityAccess::class)
      ->addArgument(new Reference('entity_type.manager'))
      ->addArgument(new Reference('workspaces.manager'))
      ->addArgument(new Reference('workspaces.information'));

    $container->register('Drupal\workspaces\EntityOperations', \Drupal\wse\Core\EntityOperations::class)
      ->addArgument(new Reference('entity_type.manager'))
      ->addArgument(new Reference('workspaces.manager'))
      ->addArgument(new Reference('workspaces.association'))
      ->addArgument(new Reference('workspaces.information'))
      ->addArgument(new Reference('typed_data_manager'));

    $container->register('Drupal\workspaces\EntityTypeInfo', \Drupal\wse\Core\EntityTypeInfo::class)
      ->addArgument(new Reference('workspaces.information'));

    $container->register('Drupal\workspaces\FormOperations', \Drupal\wse\Core\FormOperations::class)
      ->addArgument(new Reference('workspaces.manager'))
      ->addArgument(new Reference('event_dispatcher'));

    $container->register('Drupal\workspaces\ViewsQueryAlter', \Drupal\wse\Core\ViewsQueryAlter::class)
      ->addArgument(new Reference('entity_type.manager'))
      ->addArgument(new Reference('entity_field.manager'))
      ->addArgument(new Reference('workspaces.manager'))
      ->addArgument(new Reference('views.views_data'))
      ->addArgument(new Reference('plugin.manager.views.join'))
      ->addArgument(new Reference('language_manager'));

    if (!$container->hasDefinition('workspaces.information')) {
      $container->register('workspaces.information', \Drupal\wse\Core\WorkspaceInformation::class)
        ->addArgument(new Reference('entity_type.manager'))
        ->addArgument(new Reference('workspaces.association'));
    }

    if (!$container->hasDefinition('workspaces.default_safe_forms')) {
      $container->register('workspaces.default_safe_forms', \Drupal\wse\Core\EventSubscriber\DefaultWorkspaceSafeForms::class)
        ->addTag('event_subscriber');
    }

    if (!$container->hasDefinition('workspaces.controller.entity_form')) {
      $container->register('workspaces.controller.entity_form', WseHtmlEntityFormController::class)
        ->setDecoratedService('controller.entity_form')
        ->setPublic(FALSE)
        ->addArgument(new Reference('workspaces.controller.entity_form.inner'))
        ->addArgument(new Reference('workspaces.manager'))
        ->addArgument(new Reference('workspaces.information'))
        ->addArgument(new Reference('typed_data_manager'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function alter(ContainerBuilder $container) {
    if ($container->hasDefinition('workspaces.manager')) {
      $definition = $container->getDefinition('workspaces.manager');
      $definition->setClass(\Drupal\wse\Core\WorkspaceManager::class);
    }

    if ($container->hasDefinition('workspaces.negotiator.query_parameter')) {
      $definition = $container->getDefinition('workspaces.negotiator.query_parameter');
      $definition->setClass(WseQueryParameterWorkspaceNegotiator::class);
    }
  }

}
