<?php

namespace Drupal\Tests\wse\Functional;

use Drupal\Core\Database\Database;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\Tests\UiHelperTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\workspaces\Functional\WorkspaceTestUtilities;

/**
 * Move to anyther workspace test.
 *
 * @group wse
 */
class MoveToAnotherWorkspaceTest extends BrowserTestBase {

  use UiHelperTrait;
  use WorkspaceTestUtilities;
  use ContentTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'field_ui',
    'node',
    'taxonomy',
    'toolbar',
    'user',
    'options',
    'workspaces',
    'wse',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalPlaceBlock('local_actions_block');
  }

  /**
   * Test callback.
   */
  public function testMoveToAnotherWorkspaceEntityOperation() {
    // Login and create a content type.
    $this->drupalLogin($this->rootUser);
    $this->drupalCreateContentType(['type' => 'article', 'label' => 'Article']);
    $this->setupWorkspaceSwitcherBlock();

    // Create a workspace.
    $storage = \Drupal::entityTypeManager()->getStorage('workspace');
    $this->createWorkspaceThroughUi('Source', 'source');
    $source_workspace = $storage->loadByProperties(['label' => 'Source']);
    $source_workspace = reset($source_workspace);
    $this->createworkspacethroughui('Destination', 'destination');
    $destination_workspace = $storage->loadByProperties(['label' => 'Destination']);
    $destination_workspace = reset($destination_workspace);

    // Create a node in the workspace.
    $original_node = $this->createNodeThroughUi('Spock', 'article');
    $workspace_association = \Drupal::service('workspaces.association');
    $source_tracked = $workspace_association->getTrackedEntities($source_workspace->id());
    $this->assertEquals(TRUE, $source_tracked['node'][2] === $original_node->id());

    // Move node to another workspace.
    $this->drupalGet(Url::fromRoute('entity.node.move_to_workspace', [
      'node' => $original_node->id(),
      'source_workspace' => $source_workspace->id(),
    ])->toString());
    $this->submitForm([
      'target_workspace' => $destination_workspace->id(),
    ], 'Confirm');

    $this->assertEmpty($workspace_association->getTrackedEntities($source_workspace->id()));
    $destination_tracked = $workspace_association->getTrackedEntities($destination_workspace->id());
    $this->assertEquals(TRUE, $destination_tracked['node'][2] === $original_node->id());
  }

}
